/*
 * Linux_BIOSElement.c
 *
 * (C) Copyright IBM Corp. 2003
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author:       Viktor Mihajlovski <mihajlov@de.ibm.com>  
 * Contributors: James Jeffers <jamesjef@us.ibm.com>
 *
 * Interface Type : Common Manageability Programming Interface ( CMPI )
 *
 * Description: BIOS Element. Contains BIOS Information.
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cmpidt.h"
#include "cmpift.h"
#include "cmpimacs.h"

#include "dmiinfo.h"

unsigned char CMPI_true=1;
unsigned char CMPI_false=0;

CMPIBroker * _broker;

/* ---------------------------------------------------------------------------*/
/* private declarations                                                       */
#ifdef DEBUG
    int _debug = 1;
#else
    int _debug = 0;
#endif

static char * _ClassName     = "Linux_BIOSElement";

static CMPIObjectPath *makeObjectPath(DMI_BIOSELEMENT *be,CMPIObjectPath *op);
static CMPIInstance *makeInstance(DMI_BIOSELEMENT *be,CMPIObjectPath *op);

/* ---------------------------------------------------------------------------*/
/*                      Instance Provider Interface                           */
/* ---------------------------------------------------------------------------*/


CMPIStatus Linux_BIOSElementCleanup( CMPIInstanceMI * mi, 
  CMPIContext * ctx) {
  cimdmi_term();
  CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_BIOSElementEnumInstanceNames( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref) { 
  CMPIObjectPath *op;
  DMI_BIOSELEMENT *bioselem;
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI EnumInstanceNames()\n", _ClassName );

  bioselem=cimdmi_getBiosElement();
  if (bioselem) {
    op = makeObjectPath(bioselem,ref);
    if (op) {
      CMReturnObjectPath(rslt,op);
    } else {
      rc.rc=CMPI_RC_ERR_NOT_FOUND;
      rc.msg=CMNewString(_broker,
			 "Failed to extract BIOS Product Information",NULL);
    }
  }else {
    rc.rc=CMPI_RC_ERROR;
    rc.msg=CMNewString(_broker,"Failed to locate BIOS",NULL);
  }

  CMReturnDone( rslt );
  return rc;
}

CMPIStatus Linux_BIOSElementEnumInstances( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref, 
  char ** properties) { 
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  CMPIInstance *inst;
  DMI_BIOSELEMENT *bioselem;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI EnumInstances()\n", _ClassName );

  bioselem=cimdmi_getBiosElement();
  if (bioselem) {
    inst = makeInstance(bioselem,ref);
    if (inst) {
      CMReturnInstance(rslt,inst);
    } else {
      if( _debug )
	fprintf( stderr, "--- %s.c : CMPI EnumInstances() not found\n",
		 _ClassName );
      rc.rc=CMPI_RC_ERR_NOT_FOUND;
      rc.msg=CMNewString(_broker,
			 "Failed to extract BIOS Product Information",NULL);
    }
  } else {
    if( _debug )
      fprintf( stderr, "--- %s.c : CMPI EnumInstances() error\n",
		 _ClassName );
    rc.rc=CMPI_RC_ERROR;
    rc.msg=CMNewString(_broker,"Failed to locate BIOS",NULL);
  }

  CMReturnDone( rslt );
  return rc;
}

CMPIStatus Linux_BIOSElementGetInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop, 
  char ** properties) {
  CMPIInstance *inst;
  DMI_BIOSELEMENT *bioselem;
  CMPIStatus rc = {CMPI_RC_OK, NULL};

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI GetInstance()\n", _ClassName );

  bioselem=cimdmi_getBiosElement();
  if (bioselem) {
    inst = makeInstance(bioselem,cop);
    if (inst) {
      CMReturnInstance(rslt,inst);
    } else {
      if( _debug )
	fprintf( stderr, "--- %s.c : CMPI EnumInstances() not found\n",
		 _ClassName );
      rc.rc=CMPI_RC_ERR_NOT_FOUND;
      rc.msg=CMNewString(_broker,
			 "Failed to extract BIOS Product Information",NULL);
    }
  } else {
    if( _debug )
      fprintf( stderr, "--- %s.c : CMPI EnumInstances() error\n",
		 _ClassName );
    rc.rc=CMPI_RC_ERROR;
    rc.msg=CMNewString(_broker,"Failed to locate BIOS",NULL);
  }

  CMReturnDone(rslt);
  return rc;
}

CMPIStatus Linux_BIOSElementCreateInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop, 
  CMPIInstance * ci) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI CreateInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSElementSetInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop,
  CMPIInstance * ci, 
  char **properties) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI SetInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSElementDeleteInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop) {
  CMPIStatus rc = {CMPI_RC_OK, NULL}; 
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI DeleteInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSElementExecQuery( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref, 
  char * lang, 
  char * query) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI ExecQuery()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}



/* ---------------------------------------------------------------------------*/
/*                              private methods                               */
/* ---------------------------------------------------------------------------*/
static CMPIObjectPath *makeObjectPath(DMI_BIOSELEMENT *be,CMPIObjectPath *op)
{
  CMPIObjectPath * iop=NULL;

  iop = CMNewObjectPath(_broker,CMGetCharPtr(CMGetNameSpace(op,NULL)),_ClassName,NULL);
  if (iop) {
    CMSetNameSpaceFromObjectPath(iop,op);
    CMAddKey(iop,"Name",
	     be->dmi_Name,CMPI_chars);
    CMAddKey(iop,"Version",
	     be->dmi_Version,CMPI_chars);
    CMAddKey(iop,"SoftwareElementID",
	     be->dmi_SWEID,CMPI_chars);
    CMAddKey(iop,"SoftwareElementState",
	     &be->dmi_SWEState,CMPI_uint16);
    CMAddKey(iop,"TargetOperatingSystem",
	     &be->dmi_TargetOS,CMPI_uint16);
  }
  return iop;
}

static CMPIInstance *makeInstance(DMI_BIOSELEMENT *be,CMPIObjectPath *op)
{
  CMPIObjectPath * iop=NULL;
  CMPIInstance   * inst=NULL;
  CMPIDateTime   * dt;
  unsigned long long longval;


  iop = CMNewObjectPath(_broker,CMGetCharPtr(CMGetNameSpace(op,NULL)),_ClassName,NULL);
  if (iop) {
	if( _debug )
	    fprintf( stderr, "--- %s.c : CMPI makeInstance() iop \n",_ClassName);	
    CMSetNameSpaceFromObjectPath(iop,op);
    inst = CMNewInstance(_broker,iop,NULL);
  }
  if (inst) {
    CMSetProperty(inst,"Name",
		  be->dmi_Name,CMPI_chars);
    CMSetProperty(inst,"Version",
		  be->dmi_Version,CMPI_chars);
    CMSetProperty(inst,"SoftwareElementID",
		  be->dmi_SWEID,CMPI_chars);
    CMSetProperty(inst,"SoftwareElementState",
		  &be->dmi_SWEState,CMPI_uint16);
    CMSetProperty(inst,"TargetOperatingSystem",
		  &be->dmi_TargetOS,CMPI_uint16);    
    CMSetProperty(inst,"Manufacturer",
		  be->dmi_Manufacturer,CMPI_chars);
    CMSetProperty(inst,"PrimaryBIOS",
		  &CMPI_true,CMPI_boolean);
    longval=be->dmi_StartingAddress;
    CMSetProperty(inst,"LoadedStartingAddress",
		  &longval,CMPI_uint64);
    longval=be->dmi_EndingAddress;
    CMSetProperty(inst,"LoadedEndingAddress",
		  &longval,CMPI_uint64);
    longval=be->dmi_ReleaseDate;
    dt = CMNewDateTimeFromBinary(_broker,longval*1000000,0,NULL);
    CMSetProperty(inst,"ReleaseDate",
		  &dt,CMPI_dateTime);
  }
  return inst;
}

/* ---------------------------------------------------------------------------*/
/*                              Provider Factory                              */
/* ---------------------------------------------------------------------------*/

CMInstanceMIStub( Linux_BIOSElement, Linux_BIOSElement, _broker, cimdmi_init());

/* ---------------------------------------------------------------------------*/
/*                    end of Linux_BIOSElement                      */
/* ---------------------------------------------------------------------------*/

